﻿using System;
using System.Threading.Tasks;
using Windows.Globalization;
using Windows.Graphics.Imaging;
using Windows.Media.Ocr;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;

namespace OCRSample.Pages
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class WelcomePage : Page
    {
        /// <summary>
        /// 전역 SoftwareBitmap
        /// </summary>
        private SoftwareBitmap _bitmap;

        public WelcomePage()
        {
            this.InitializeComponent();
        }
        /// <summary>
        /// 오픈 파일 버튼 클릭
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void OpenFile_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            //이미지 파일을 선택하기 위해서 파일 피커를 사용합니다.
            var picker = new FileOpenPicker()
            {
                SuggestedStartLocation = PickerLocationId.PicturesLibrary,
                FileTypeFilter = { ".jpg", ".jpeg", ".png" },
            };
            //선택된 파일하나를 가지고 옵니다.
            var file = await picker.PickSingleFileAsync();
            if (file != null)
            {
                //화면에 출력된 텍스트를 지웁니다.
                ClearResults();
                //이미지를 로딩합니다.
                await LoadImage(file);
            }

        }
        /// <summary>
        /// 이미지 로딩
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        private async Task LoadImage(StorageFile file)
        {
            //전달받은 file을 읽기 전용으로 열고, 스트림을 얻습니다.
            using (var stream = await file.OpenAsync(FileAccessMode.Read))
            {
                //비트맵 디코더를 생성
                var decoder = await BitmapDecoder.CreateAsync(stream);

                //디코더를 이용해서 SoftwareBitmap을 생성합니다. OCR 기능을 이용하기 위해서는 SoftwareBitmap이 필요합니다.
                //https://msdn.microsoft.com/en-us/library/windows/apps/windows.graphics.imaging.softwarebitmap.aspx
                //Universal, introduced version 10.0.10240.0 이상 버전에서만 사용가능한 클래스로 
                //이미징 API를 사용하기 쉽게 만든 비트맵입니다.
                _bitmap = await decoder.GetSoftwareBitmapAsync(BitmapPixelFormat.Bgra8, BitmapAlphaMode.Premultiplied);

                //화면에 출력하기위해 WriteableBitmap을 만듭니다.
                var imgSource = new WriteableBitmap(_bitmap.PixelWidth, _bitmap.PixelHeight);
                _bitmap.CopyToBuffer(imgSource.PixelBuffer);
                //화면 컨크롤에 출력합니다.
                PreviewImage.Source = imgSource;
            }
        }

        private void ClearResults()
        {
            
        }

        private async void Extract_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            if (_bitmap == null)
            {
                await MsgBox("이미지가 없지 말입니다~.");
                return;
            }
            // OcrEngine이 지원하는 이미지 사이즈인지 확인
            if (_bitmap.PixelWidth > OcrEngine.MaxImageDimension || _bitmap.PixelHeight > OcrEngine.MaxImageDimension)
            {
                await MsgBox("이미지가 너무 커서 않되지 말입니다~.");
                return;
            }

            //드디어 출격!!
            ////UserProfile.GlobalizationPreferences.Languages에 정의되어 있는 가장 첫번째 언어를 사용하도록 합니다.
            //var ocrEngine = OcrEngine.TryCreateFromUserProfileLanguages();
            //if (ocrEngine == null)
            //{
            //    await MsgBox("지원 가능한 언어가 없지 말입니다~.");
            //    return;
            //}

            //한국어 인식하도록 지정
            var ocrEngine = OcrEngine.TryCreateFromLanguage(new Language("ko"));
            if (ocrEngine == null)
            {
                await MsgBox("지원 가능한 언어가 없지 말입니다~.");
                return;
            }

            // 이미지에서 택스트를 인식합니다.
            var ocrResult = await ocrEngine.RecognizeAsync(_bitmap);

            // 인식된 텍스트를 텍스트블럭에 출력
            Extracted.Text = ocrResult.Text;

        }
        /// <summary>
        /// 메시지 박스 출력
        /// </summary>
        /// <param name="message"></param>
        /// <returns></returns>
        private async Task MsgBox(string message)
        {
            await new MessageDialog(message).ShowAsync();
        }
    }
}
